#
# Created on Wed Feb 15 2023
#
# Copyright (c) 2023 Gddi
#
import json
from typing import List

from data_client.dataset.value_object.coco.coco_annotation import \
    CocoAnnotation
from data_client.dataset.value_object.rgbd.rgbd_annotation import \
    RgbdAnnotation
from data_client.dataset.value_object.rgbd.rgbd_raw_data import RgbdRawData
from RawData.Rgbd.RgbdData import RgbdData

# annotation --------------------------------------------------------------

def parse_bounding_box(filepath : str, label_index : dict) -> List:
    result = []
    with open(filepath, 'r') as file:
        for bbox_in in json.load(file)["BoundingBoxList"]:
            label_id_str = bbox_in["LabelId"]
            bbox_out = [bbox_in["X"], bbox_in["Y"], bbox_in["Width"], bbox_in["Height"], label_index[label_id_str]]
            result.append(bbox_out)
    return result

def parse_coco_annotation(filepath : str, label_index : dict) -> List[CocoAnnotation]:
    res = []
    with open(filepath, 'r') as file:
        json_data = json.load(file)["AnnoData"]
        for data in json_data:
            res.append(CocoAnnotation(json_data=data, label_index=label_index))
    return res


def parse_rgbd_annotation(filepath: str, label_index : dict) -> List[RgbdAnnotation]:
    res = []
    with open(filepath, 'r') as file:
        json_data = json.load(file)["BoundingBoxList"]
        for data in json_data:
            res.append(RgbdAnnotation(json_data=data, label_index=label_index))
    return res

# raw data --------------------------------------------------------------

def parse_rgbd_raw_data(filepath: str) -> RgbdRawData:
    result = {}
    with open(filepath, 'rb') as file:
        buf = file.read()
        rgbdData = RgbdData.GetRootAsRgbdData(buf, 0)
        return RgbdRawData(rgbdData)