#
# Created on Mon Feb 13 2023
#
# Copyright (c) 2023 Gddi
#

import faulthandler
import sys
import unittest
from typing import List

import cv2
import numpy as np
import torch.utils.data as Data
from loguru import logger
from tqdm import tqdm

from data_client.dataset import DatasetConfig, GddiDatasetV1
from data_client.dataset.value_object.rgbd.rgbd_annotation import \
    RgbdAnnotation
from data_client.dataset.value_object.rgbd.rgbd_raw_data import RgbdRawData
from data_client.utils import const


class FlowerDataset(GddiDatasetV1):
    def __getitem__(self, index):
        raw_data, anno_map = super().__getitem__(index)
        img = raw_data.data
        resized_img = cv2.resize(
            src = img,
            dsize = (100, 100),
            interpolation=cv2.INTER_LINEAR
        ).astype(np.uint8)
        anno = list(anno_map.values())[0]
        print(anno)
        return resized_img ,anno

class HandDectDataset(GddiDatasetV1):
    def __getitem__(self, index):
        raw_data, anno_map = super().__getitem__(index)
        img = raw_data.data
        resized_img = cv2.resize(
            src = img,
            dsize = (100, 100),
            interpolation=cv2.INTER_LINEAR
        ).astype(np.uint8)
        anno = list(anno_map.values())[0]
        return resized_img ,anno

class CocoDataset(GddiDatasetV1):
    def __getitem__(self, index):
        img, anno_map = super().__getitem__(index)
        logger.debug(f"raw data id: {img.id}")
        resized_img = cv2.resize(
            src = img.data,
            dsize = (100, 100),
            interpolation=cv2.INTER_LINEAR
        ).astype(np.uint8)
        anno = list(anno_map.values())[0]
        print(len(anno))
        print("id: {}, kpt {}, area {}".format(anno[0].id, anno[0].key_points, anno[0].area))
        return resized_img ,1

class OcrDataset(GddiDatasetV1):
    def __getitem__(self, index):
        raw_data, anno_map = super().__getitem__(index)
        img = raw_data.data
        resized_img = cv2.resize(
            src = img,
            dsize = (100, 100),
            interpolation=cv2.INTER_LINEAR
        ).astype(np.uint8)
        anno : list[RgbdAnnotation] = list(anno_map.values())[0]
        print(anno)
        return resized_img ,1

class RgbdDataset(GddiDatasetV1):
    def __getitem__(self, index):
        raw_data, anno_map = super().__getitem__(index)
        logger.debug(raw_data.image)
        logger.debug(raw_data.depth)
        logger.debug(raw_data.extrinsics)
        logger.debug(raw_data.intrinsics)
        anno = list(anno_map.values())[0]
        if (len(anno) > 0):
            logger.debug(anno[0].__dict__)
        return 1 ,1
    
class SegMasksDataset(GddiDatasetV1):
    def __getitem__(self, index):
        raw_data, anno_map = super().__getitem__(index)
        anno = list(anno_map.values())[0]
        logger.debug(f'raw data: {raw_data.data}')
        logger.info(f'raw data shape: {raw_data.data.shape}')
        logger.debug(f'annotation: {anno}')
        logger.debug(f'anno is zero: {np.all(anno == 0)}')
        return 1 ,1

class TestDataset(unittest.TestCase):
    """dataset unit test stubs"""

    def setUp(self):
        pass

    def tearDown(self):
        pass

    @unittest.skip("reason for skipping")
    def testFlowerClassification(self):
        faulthandler.enable()
        logger.remove()
        logger.add(sys.stderr, level="DEBUG")
        
        """Test GddiDatasetV1"""
        dataset_config = DatasetConfig(
            delete_raw_data_with_no_annotation = True
        )
        # dataset = FlowerDataset(["16bdffd9-1b78-4b2f-b26c-fb3c703e1a29", "0ba6899f-2c6b-48ab-8b0e-7c47e5f04cc6"])
        dataset = FlowerDataset(
            data_view_id_list = ["6b21b10e-4c8d-476b-898e-de2760d8c4c7", "61610fb8-55cb-45bd-90c1-9926b1150361"], 
            config = dataset_config
        )
       
        print(dataset.get_label_map())
        loader = Data.DataLoader(
            dataset=dataset,
            batch_size=10,
            shuffle=True,
            num_workers=0,
        )
        for epoch in range(1):
            num = 0
            for raw_data, annotation in loader:
                num = num + 1
                # print('Epoch: {} | num: {} | raw_data: {} | annotation: {}'.format(epoch,num, raw_data, annotation))

    @unittest.skip("reason for skipping")
    def testHandDetection(self):
        faulthandler.enable()
        logger.remove()
        logger.add(sys.stderr, level="DEBUG")
        
        dataset_config = DatasetConfig(
            delete_raw_data_with_no_annotation = True
        )
        
        dataset = HandDectDataset(
            data_view_id_list = ["4366d960-376d-405a-8038-34c2064c5273", "0891f334-25c3-4d35-a613-762519587a91"], 
            config = dataset_config
        )
        print(dataset.get_label_map())
        loader = Data.DataLoader(
            dataset=dataset,
            batch_size=10,
            shuffle=True,
            num_workers=0,
        )
        for epoch in range(1):
            num = 0
            for raw_data, annotation in loader:
                num = num + 1
                print('Epoch: {} | num: {} | raw_data: {} | annotation: {}'.format(epoch,num, raw_data, annotation))
    
    @unittest.skip("reason for skipping")
    def testCocodataset(self):
        faulthandler.enable()
        logger.remove()
        logger.add(sys.stderr, level="DEBUG")
        
        dataset_config = DatasetConfig(
            delete_raw_data_with_no_annotation = True
        )
        
        dataset = CocoDataset(
            #train
            #data_view_id_list = ["b3518774-b0bb-4581-b31f-29a3ace7b085", "02eabb5c-d458-4eb6-b704-06673101917f"], 
            #valid
            data_view_id_list = ["03ffeed3-a694-4e9a-b2a9-0c37c7fa4bdd", "ce26ea46-639c-47a8-9652-d2adfe319b2b"], 
            config = dataset_config
        )
        print(dataset.get_label_map())
        print(dataset.get_annotation_template_map())
        loader = Data.DataLoader(
            dataset=dataset,
            batch_size=10,
            shuffle=True,
            num_workers=0,
        )
        for epoch in range(1):
            num = 0
            for raw_data, annotation in loader:
                num = num + 1
                #print('Epoch: {} | num: {} | raw_data: {} | annotation: {}'.format(epoch,num, raw_data, annotation))
    
    @unittest.skip("reason for skipping")
    def testOcrdataset(self):
        faulthandler.enable()
        logger.remove()
        logger.add(sys.stderr, level="DEBUG")
        
        dataset_config = DatasetConfig(
            delete_raw_data_with_no_annotation = True
        )
        
        dataset = OcrDataset(
            # train
            #data_view_id_list = ["35552fdc-1f04-4f7d-bf0b-86b156410af7", "3db85b65-3515-476b-8126-7b51845c706a"], 
            # val
            data_view_id_list = ["4a5a48b3-f44b-4e30-9300-a86512535779", "800c4a0d-1d9a-4975-91df-3ce5df74d395"], 
            config = dataset_config
        )
        print(dataset.get_word_list_map())
        loader = Data.DataLoader(
            dataset=dataset,
            batch_size=10,
            shuffle=True,
            num_workers=0,
        )
        for epoch in range(1):
            num = 0
            for raw_data, annotation in loader:
                num = num + 1
                #print('Epoch: {} | num: {} | raw_data: {} | annotation: {}'.format(epoch,num, raw_data, annotation))

    @unittest.skip("reason for skipping")
    def testRgbddataset(self):
        faulthandler.enable()
        logger.remove()
        logger.add(sys.stderr, level="DEBUG")
        
        dataset_config = DatasetConfig(
            delete_raw_data_with_no_annotation = True
        )
        
        dataset = RgbdDataset(
            # train
            #data_view_id_list = ["2c39143e-918c-4377-b1ca-1e34618a7f4d", "dda8bb8c-3eda-4bfb-9488-6f5f5c19ead6"], 
            # val
            data_view_id_list = ["12160eac-179d-4f2a-8055-e79f8ccbba26", "04550946-3add-47a2-9011-a329a0c2b6aa"], 
            config = dataset_config
        )
        loader = Data.DataLoader(
            dataset=dataset,
            batch_size=10,
            shuffle=True,
            num_workers=0,
        )
        for epoch in range(1):
            num = 0
            for raw_data, annotation in tqdm(loader):
                num = num + 1
                #print('Epoch: {} | num: {} | raw_data: {} | annotation: {}'.format(epoch,num, raw_data, annotation))
    
    @unittest.skip("reason for skipping")
    def testSegMasksdataset(self):
        faulthandler.enable()
        logger.remove()
        logger.add(sys.stderr, level="INFO")
        
        dataset_config = DatasetConfig(
            delete_raw_data_with_no_annotation = False
        )
        
        dataset = SegMasksDataset(
            # train
            data_view_id_list = ["8384e1b9-16ec-42ff-8141-8368973b2e5c", "8faaa853-a5a8-4e6f-9f27-a10abffbce1b"], 
            # val
            #data_view_id_list = ["1da7e802-15d6-4bec-9d69-1d0feb09315b", "0f87a257-10ab-4763-b8d4-d70138cb51d3"], 
            config = dataset_config
        )
        loader = Data.DataLoader(
            dataset=dataset,
            batch_size=10,
            shuffle=True,
            num_workers=0,
        )
        for epoch in range(1):
            num = 0
            for raw_data, annotation in tqdm(loader):
                num = num + 1
                #print('Epoch: {} | num: {} | raw_data: {} | annotation: {}'.format(epoch,num, raw_data, annotation))
    
if __name__ == '__main__':
    unittest.main()
